import numpy as np

def repeat_center(n, repeat):
    """
    Create the center-point portion of a design matrix
    
    Parameters
    ----------
    n : int
        The number of factors in the original design
    repeat : int
        The number of center points to repeat
    
    Returns
    -------
    mat : 2d-array
        The center-point portion of a design matrix (elements all zero).
    
    Example
    -------
    ::
    
        >>> repeat_center(3, 2)
        array([[ 0.,  0.,  0.],
               [ 0.,  0.,  0.]])
       
    """
    return np.zeros((repeat, n))


import numpy as np
from doe_factorial import ff2n
from doe_star import star
from doe_union import union
from doe_repeat_center import repeat_center

def ccdesign(n, center=1, face='circumscribed'):
    """
    Central composite design
    
    Parameters
    ----------
    n : int
        The number of factors in the design.
    
    Optional
    --------
    center : int
        The number of center points (default: 0)
    face : str
        The relation between the start points and the corner (factorial) points.
        There are three options for this input:
        
        1. 'circumscribed' or 'ccc': This is the original form of the central
           composite design. The star points are at some distance ``alpha``
           from the center, based on the properties desired for the design.
           The start points establish new extremes for the low and high
           settings for all factors. These designs have circular, spherical,
           or hyperspherical symmetry and require 5 levels for each factor.
           Augmenting an existing factorial or resolution V fractional 
           factorial design with star points can produce this design.
        
        2. 'inscribed' or 'cci': For those situations in which the limits
           specified for factor settings are truly limits, the CCI design
           uses the factors settings as the star points and creates a factorial
           or fractional factorial design within those limits (in other words,
           a CCI design is a scaled down CCC design with each factor level of
           the CCC design divided by ``alpha`` to generate the CCI design).
           This design also requires 5 levels of each factor.
        
        3. 'faced' or 'ccf': In this design, the star points are at the center
           of each face of the factorial space, so ``alpha`` = +/-1. This 
           variety requires 3 levels of each factor. Augmenting an existing 
           factorial or resolution V design with appropriate star points can 
           also produce this design.
    
        Note 1: The value for ``alpha`` is automatically calculated to maintain 
        rotatability.
        
        Note 2: Fractional factorial designs are not (yet) available here.
        
    Returns
    -------
    mat : 2d-array
        The design matrix with coded levels -1 and 1
    
    Example
    -------
    ::
    
        >>> ccdesign(3)
        array([[-1.        , -1.        , -1.        ],
               [ 1.        , -1.        , -1.        ],
               [-1.        ,  1.        , -1.        ],
               [ 1.        ,  1.        , -1.        ],
               [-1.        , -1.        ,  1.        ],
               [ 1.        , -1.        ,  1.        ],
               [-1.        ,  1.        ,  1.        ],
               [ 1.        ,  1.        ,  1.        ],
               [-1.68179283,  0.        ,  0.        ],
               [ 1.68179283,  0.        ,  0.        ],
               [ 0.        , -1.68179283,  0.        ],
               [ 0.        ,  1.68179283,  0.        ],
               [ 0.        ,  0.        , -1.68179283],
               [ 0.        ,  0.        ,  1.68179283],
               [ 0.        ,  0.        ,  0.        ]])
       
    """
    H1 = ff2n(n)
    H2 = star(n)
    
    if face.lower() in ('circumscribed', 'ccc'):
        alpha = (H1.shape[0])**0.25
    elif face.lower() in ('inscribed', 'cci'):
        alpha = 1
        H1 /= (H1.shape[0])**0.25
    elif face.lower() in ('faced', 'ccf'):
        alpha = 1
    else:
        raise Exception, 'Invalid input for "face": {:}'.format(face)
        
    H = union(H1, alpha*H2)
    H = np.c_[H.T, repeat_center(nb_var, center).T].T
    return H


import numpy as np

def fullfact(levels):
    """
    Create a general full-factorial design
    
    Parameters
    ----------
    levels : array-like
        An array of integers that indicate the number of levels of each input
        design factor.
    
    Returns
    -------
    mat : 2d-array
        The design matrix with coded levels 1 to k for a k-level factor
    
    Example
    -------
    ::
    
        >>> fullfact([2, 4, 3])
        array([[ 1.,  1.,  1.],
               [ 2.,  1.,  1.],
               [ 1.,  2.,  1.],
               [ 2.,  2.,  1.],
               [ 1.,  3.,  1.],
               [ 2.,  3.,  1.],
               [ 1.,  4.,  1.],
               [ 2.,  4.,  1.],
               [ 1.,  1.,  2.],
               [ 2.,  1.,  2.],
               [ 1.,  2.,  2.],
               [ 2.,  2.,  2.],
               [ 1.,  3.,  2.],
               [ 2.,  3.,  2.],
               [ 1.,  4.,  2.],
               [ 2.,  4.,  2.],
               [ 1.,  1.,  3.],
               [ 2.,  1.,  3.],
               [ 1.,  2.,  3.],
               [ 2.,  2.,  3.],
               [ 1.,  3.,  3.],
               [ 2.,  3.,  3.],
               [ 1.,  4.,  3.],
               [ 2.,  4.,  3.]])

    """
    n = len(levels)  # number of factors
    nb_lines = np.prod(levels)  # number of trial conditions
    H = np.zeros((nb_lines, n))
    
    level_repeat = 1
    range_repeat = np.prod(levels)
    for i in xrange(n):
        range_repeat /= levels[i]
        lvl = []
        for j in xrange(levels[i]):
            lvl += [j]*level_repeat
        rng = lvl*range_repeat
        level_repeat *= levels[i]
        H[:, i] = rng
     
    return H
    
def ff2n(n):
    """
    Create a 2-Level full-factorial design
    
    Parameters
    ----------
    n : int
        The number of factors in the design.
    
    Returns
    -------
    mat : 2d-array
        The design matrix with coded levels -1 and 1
    
    Example
    -------
    ::
    
        >>> ff2n(3)
        array([[-1., -1., -1.],
               [ 1., -1., -1.],
               [-1.,  1., -1.],
               [ 1.,  1., -1.],
               [-1., -1.,  1.],
               [ 1., -1.,  1.],
               [-1.,  1.,  1.],
               [ 1.,  1.,  1.]])
    
    """
    H = fullfact([2]*n)
    H[H==0] = -1
    H[H==1] = 1
    
    return H


import numpy as np

def star(n):
    """
    Create the star points of various design matrices
    
    Parameters
    ----------
    n : int
        The number of variables in the design
    
    Returns
    -------
    mat : 2d-array
        The unit-star-point portion of the design matrix (i.e. at +/- 1)
    
    Example
    -------
    ::
    
        >>> star(3)
        array([[-1.,  0.,  0.],
               [ 1.,  0.,  0.],
               [ 0., -1.,  0.],
               [ 0.,  1.,  0.],
               [ 0.,  0., -1.],
               [ 0.,  0.,  1.]])
               
    """
    H = np.zeros((2*nb_var, nb_var))
    for i in xrange(nb_var):
        H[2*i:2*i+2, i] = [-1, 1]
    return H


import numpy as np

def union(H1, H2):
    """
    Join two matrices by stacking them on top of each other.
    
    Parameters
    ----------
    H1 : 2d-array
        The matrix that goes on top of the new matrix
    H2 : 2d-array
        The matrix that goes on bottom of the new matrix
    
    Returns
    -------
    mat : 2d-array
        The new matrix that contains the rows of ``H1`` on top of the rows of
        ``H2``.
    
    Example
    -------
    ::
    
        >>> union(np.eye(2), -np.eye(2))
        array([[ 1.,  0.],
               [ 0.,  1.],
               [-1.,  0.],
               [ 0., -1.]])
               
    """
    H = np.r_[H1, H2]
    return H


import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='pyDOE',
    version='0.1 alpha',
    author='Abraham Lee',
    author_email='tisimst@gmail.com',
    description='Design of experiments for Python',
    url='https://github.com/tisimst/pyDOE',
    license='BSD License',
    long_description=read('README.rst'),
    packages=['pyDOE'],
    install_requires=['numpy'],
    keywords=[
        'DOE',
        'design of experiments',
        'experimental design',
        'optimization',
        'statistics',
        'python'
        ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Topic :: Education',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
        ]
    )
